//IndustryPackModule.cc

/********************COPYRIGHT NOTIFICATION**********************************
  This software was developed under a United States Government license
  described on the COPYRIGHT_UniversityOfChicago file included as part
  of this distribution.
  ***************************************************************************
  */
/*
  Current Author: Thomas Coleman
  Original Author: Jim Kowalkowski (for Hideos)
  Date: 09JUL98
  */

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <intLib.h>
 
#include "DLList.h"
#include "IndustryPackCarrier.h"
#include "IndustryPackModule.h"

static DLList *pIndustryPackModuleList = 0;

// ------------------------------------------------------------------------
// constructor for an IndustryPack module

IndustryPackModule::IndustryPackModule
(const char *name, IndustryPackCarrier *pCarrier, IndustryPackSite *pSite)
{
  pIpCarrier = pCarrier;
  pIpSite = pSite;
  moduleName = name;
  
  memBaseID = pIpSite->getMemBaseID();
  memBaseIO = pIpSite->getMemBaseIO();

  memBaseIP = 0;
  memSizeIP = 0;

  //  printf("IndustryPackModule::IndustryPackModule moduleName %s\n",moduleName);
  //  printf("IndustryPackModule::IndustryPackModule memBaseID %p\n",memBaseID);
  //  printf("IndustryPackModule::IndustryPackModule memBaseIO %p\n",memBaseIO);
  //  printf("IndustryPackModule::IndustryPackModule memBaseIP %p\n",memBaseIP);
  //  printf("IndustryPackModule::IndustryPackModule memSizeIP %d\n",memSizeIP);

  if(pIndustryPackModuleList==0) pIndustryPackModuleList = new DLList;
  DLNode *node = DLList::allocNode();
  node->obj = this;
  pIndustryPackModuleList->addEnd(node);
  return;
}

IndustryPackModule *IndustryPackModule::createIndustryPackModule
(const char *moduleName, const char *carrierName, const char *siteName)
{

  // make sure module name is not already configured
  if(IndustryPackModule::find(moduleName))
    {
      printf("IndustryPackModule::createIndustryPackModule moduleName %s already initialized\n",moduleName);
      return(0);
    }
	
  // make sure carrier site is not already configured
  if(IndustryPackModule::find(carrierName,siteName))
    {
      printf("IndustryPackModule::createIndustryPackModule carrierName %s siteName %s already initialized\n",
	     carrierName,siteName);
      return(0);
    }
	
  // make sure carrier is present
  IndustryPackCarrier *pCarrier = IndustryPackCarrier::find(carrierName);
  if(pCarrier == 0)
    {
      printf("IndustryPackModule::createIndustryPackModule carrierName %s not found\n",
	     carrierName);
      return(0);
    }
	
  // make sure carrier site is occupied
  IndustryPackSite *pSite = pCarrier->findSite(siteName);
  if(pSite == 0)
    {
      printf("IndustryPackModule::createIndustryPackModule siteName %s not occupied\n",
	     siteName);
      return(0);
    }
	
  IndustryPackModule *pModule = new IndustryPackModule(moduleName,pCarrier,pSite);

  return(pModule);
}

IndustryPackModule *IndustryPackModule::find(const char *moduleName)
{
  if (pIndustryPackModuleList == 0) pIndustryPackModuleList = new DLList();
  DLNode *node = pIndustryPackModuleList->getFirst();
  while (node) {
    IndustryPackModule *pModule = (IndustryPackModule *)node->obj;
    if(::strcmp(moduleName,pModule->moduleName)==0) {
      return(pModule);
    }
    node = pIndustryPackModuleList->getNext(node);
  }
  return(0);
}

IndustryPackModule *IndustryPackModule::find(const char *carrierName, const char *siteName)
{
  if (pIndustryPackModuleList == 0) pIndustryPackModuleList = new DLList();
  DLNode *node = pIndustryPackModuleList->getFirst();
  while (node) {
    IndustryPackModule *pModule = (IndustryPackModule *)node->obj;
    if(::strcmp(carrierName,pModule->pIpCarrier->getCarrierName())==0 &&
       ::strcmp(siteName,pModule->pIpSite->getSiteName())==0){
      return(pModule);
    }
    node = pIndustryPackModuleList->getNext(node);
  }
  return(0);
}

volatile void *IndustryPackModule::allocMemMapIP(int size)
{
  memSizeIP = size;
  memBaseIP = pIpSite->allocMemMapIP(size);
  
  //  printf ("IndustryPackModule::allocMemMapIP memBaseIP %p memSizeIP %x\n", memBaseIP, memSizeIP);

  return (memBaseIP);
}

int IndustryPackModule::intConfig(int num)
{
    pIpSite->intConfig(num);
    return(0);
}

int IndustryPackModule::intEnable(int num)
{
  pIpSite->intEnable(num);
  return(0);
}

int IndustryPackModule::intDisable(int num)
{
    pIpSite->intDisable(num);
    return(0);
}

int IndustryPackModule::intClear(int num)
{
    pIpSite->intClear(num);
    return(0);
}

volatile IP_ID_PROM *IndustryPackModule::getMemBaseID()
{
    return(memBaseID);
}

volatile void *IndustryPackModule::getMemBaseIO()
{
    return(memBaseIO);
}

volatile void *IndustryPackModule::getMemBaseIP()
{
    return(memBaseIP);
}

int IndustryPackModule::getMemSizeIP()
{
    return(memSizeIP);
}

unsigned char IndustryPackModule::getManufacturer()
{
    return(memBaseID->manufacturer);
}

unsigned char IndustryPackModule::getModel()
{
    return(memBaseID->model);
}

unsigned char IndustryPackModule::getRevision()
{
    return(memBaseID->revision);
}

