// DataFreeList.cc

/* Author:  Marty Kraimer Date:    OCT98 */

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include "DataFreeList.h"
#include "DLList.h"

static const int nDataList=9;
static const int dataListSize[nDataList] =
    {16,32,64,128,256,512,1024,2048,4096};
static DLList *papdataList[nDataList] =
    {0,0,0,0,0,0,0,0,0};

void DataFreeList::alloc(int size,void **pmem,int *actualSize)
{
    int ind;
    for(ind=0; ind<nDataList; ind++) {
	if(size<=dataListSize[ind]) break;
    }
    if(ind>=nDataList) {
	*pmem = ::calloc(size,sizeof(char));
	if(*pmem == 0) {
	    printf("DataFreeList::alloc error\n");
	    exit(0);
	}
	*actualSize = size;
	return;
    }
    if(papdataList[ind]==0) papdataList[ind] = new DLList;
    DLList *pFreeList = papdataList[ind];
    void *data = 0;
    pFreeList->lock();
    DLNode  *node = pFreeList->getFirst();
    if(node) {
	data = node->obj;
	pFreeList->freeNode(node);
    }
    pFreeList->unlock();
    if(data==0) {
	data = ::calloc(dataListSize[ind],sizeof(char));
	if(data == 0) {
	    printf("DataFreeList::alloc error\n");
	    exit(0);
	}
    }
    *pmem = data;
    *actualSize = dataListSize[ind];
}

void DataFreeList::free(void *pmem,int actualSize)
{
    if(actualSize==0) return;
    int ind;
    for(ind=0; ind<nDataList; ind++) {
	if(actualSize<=dataListSize[ind]) break;
    }
    if(ind>=nDataList) {
	::free(pmem);
    } else {
        DLList *pFreeList = papdataList[ind];
	DLNode *node = DLList::allocNode();
	node->obj = pmem;
	pFreeList->lock();
	pFreeList->addEnd(node);
	pFreeList->unlock();
    }
}
	

